from amberelectric.model.interval import SpikeStatus
from amberelectric.configuration import Configuration
from amberelectric.api.amber_api import AmberApi
from amberelectric.rest import RESTClientObject, RESTResponse
from amberelectric.model.actual_interval import ActualInterval
from amberelectric.model.current_interval import CurrentInterval
from amberelectric.model.forecast_interval import ForecastInterval
from amberelectric.model.usage import Usage
from amberelectric.model.tariff_information import PeriodType, SeasonType
from amberelectric.model.channel import ChannelType
import pytest
from datetime import date, datetime
from dateutil.tz import tzoffset, tzutc


class MockRestResponse(RESTResponse):
    def __init__(self, status, reason, data):
        self.status = status
        self.reason = reason
        self.data = data

    def getheaders(self):
        {}

    def getheader(self, name, default=None):
        return None


@pytest.fixture
def configuration():
    return Configuration(
        access_token='psk_secret_key'
    )


def test_sites_success(mocker, configuration):
    json = """
      [
        {
          "id": "01F5A5CRKMZ5BCX9P1S4V990AM",
          "nmi": "3052282872",
          "channels": [
            {
              "identifier": "E1",
              "type": "general"
            },
            {
              "identifier": "E2",
              "type": "controlledLoad"
            },
            {
              "identifier": "B1",
              "type": "feedIn"
            }
          ]
        }
      ]
    """

    def mock_sites(self, method, path, query_params=None, headers=None, body=None, post_params=None, _preload_content=True, _request_timeout=None):
        assert(method == "GET")
        assert(path == "https://api.amber.com.au/v1/sites")
        assert(query_params is None)
        assert(headers == {'Authorization': 'Bearer psk_secret_key'})
        return MockRestResponse(200, "", json.encode('utf-8'))

    mocker.patch('amberelectric.rest.RESTClientObject.request', mock_sites)

    api = AmberApi(RESTClientObject(configuration))
    sites = api.get_sites()
    assert len(sites) == 1
    site = sites[0]
    assert site.id == "01F5A5CRKMZ5BCX9P1S4V990AM"
    assert site.nmi == "3052282872"
    assert len(site.channels) == 3

    general = site.channels[0]
    assert general.identifier == "E1"
    assert general.type == ChannelType.GENERAL

    controlled_load = site.channels[1]
    assert controlled_load.identifier == "E2"
    assert controlled_load.type == ChannelType.CONTROLLED_LOAD

    feed_in = site.channels[2]
    assert feed_in.identifier == "B1"
    assert feed_in.type == ChannelType.FEED_IN


def test_prices_success(mocker, configuration):
    json = """
      [
        {
          "type": "ActualInterval",
          "duration": 5,
          "spotPerKwh": 6.12,
          "perKwh": 24.33,
          "date": "2021-05-05",
          "nemTime": "2021-05-06T12:30:00+10:00",
          "startTime": "2021-05-05T02:00:01Z",
          "endTime": "2021-05-05T02:30:00Z",
          "renewables": 45,
          "channelType": "general",
          "tariffInformation": {
            "period": "offPeak",
            "season": "summer",
            "block": 2,
            "demandWindow": true
          },
          "spikeStatus": "none"
        },
        {
          "type": "ForecastInterval",
          "duration": 5,
          "spotPerKwh": 6.12,
          "perKwh": 24.33,
          "date": "2021-05-05",
          "nemTime": "2021-05-06T12:30:00+10:00",
          "startTime": "2021-05-05T02:00:01Z",
          "endTime": "2021-05-05T02:30:00Z",
          "renewables": 45,
          "channelType": "general",
          "tariffInformation": {
            "period": "shoulder",
            "season": "winter",
            "block": 2,
            "demandWindow": true
          },
          "spikeStatus": "potential",
          "range": {
            "min": 10,
            "max": 90
          }
        },
        {
          "type": "CurrentInterval",
          "duration": 5,
          "spotPerKwh": 6.12,
          "perKwh": 24.33,
          "date": "2021-05-05",
          "nemTime": "2021-05-06T12:30:00+10:00",
          "startTime": "2021-05-05T02:00:01Z",
          "endTime": "2021-05-05T02:30:00Z",
          "renewables": 45,
          "channelType": "general",
          "tariffInformation": {
            "period": "peak",
            "season": "weekend",
            "block": 2,
            "demandWindow": true
          },
          "spikeStatus": "spike",
          "range": {
            "min": 0,
            "max": 100
          },
          "estimate": true
        }
      ]
    """

    def mock_prices(self, method, path, query_params=None, headers=None, body=None, post_params=None, _preload_content=True, _request_timeout=None):
        assert(method == "GET")
        assert(path == "https://api.amber.com.au/v1/sites/01F5A5CRKMZ5BCX9P1S4V990AM/prices")
        assert(query_params is None)
        assert(headers == {'Authorization': 'Bearer psk_secret_key'})
        return MockRestResponse(200, "", json.encode('utf-8'))

    mocker.patch('amberelectric.rest.RESTClientObject.request', mock_prices)

    api = AmberApi(RESTClientObject(configuration))
    prices = api.get_prices('01F5A5CRKMZ5BCX9P1S4V990AM')
    assert len(prices) == 3

    actual = prices[0]
    current = prices[2]
    forecast = prices[1]

    assert(actual.__class__ == ActualInterval)
    assert(actual.duration == 5)
    assert(actual.spot_per_kwh == 6.12)
    assert(actual.per_kwh == 24.33)
    assert(actual.date == date(2021, 5, 5))
    assert(actual.nem_time == datetime(2021, 5, 6, 12, 30, tzinfo=tzoffset(None, 36000)))
    assert(actual.start_time == datetime(2021, 5, 5, 2, 0, 1, tzinfo=tzutc()))
    assert(actual.end_time == datetime(2021, 5, 5, 2, 30, tzinfo=tzutc()))
    assert(actual.renewables == 45.0)
    assert(actual.channel_type == ChannelType.GENERAL)
    assert(actual.spike_status == SpikeStatus.NO_SPIKE)
    assert(actual.tariff_information.period == PeriodType.OFF_PEAK)
    assert(actual.tariff_information.season == SeasonType.SUMMER)
    assert(actual.tariff_information.block == 2)
    assert(actual.tariff_information.demand_window == True)

    assert(current.__class__ == CurrentInterval)
    assert(current.duration == 5)
    assert(current.spot_per_kwh == 6.12)
    assert(current.per_kwh == 24.33)
    assert(current.date == date(2021, 5, 5))
    assert(current.nem_time == datetime(2021, 5, 6, 12, 30, tzinfo=tzoffset(None, 36000)))
    assert(current.start_time == datetime(2021, 5, 5, 2, 0, 1, tzinfo=tzutc()))
    assert(current.end_time == datetime(2021, 5, 5, 2, 30, tzinfo=tzutc()))
    assert(current.renewables == 45.0)
    assert(current.channel_type == ChannelType.GENERAL)
    assert(current.spike_status == SpikeStatus.SPIKE)
    assert(current.tariff_information.period == PeriodType.PEAK)
    assert(current.tariff_information.season == SeasonType.WEEKEND)
    assert(current.tariff_information.block == 2)
    assert(current.tariff_information.demand_window == True)
    assert(current.range.min == 0)
    assert(current.range.max == 100)
    assert(current.estimate == True)

    assert(forecast.__class__ == ForecastInterval)
    assert(forecast.duration == 5)
    assert(forecast.spot_per_kwh == 6.12)
    assert(forecast.per_kwh == 24.33)
    assert(forecast.date == date(2021, 5, 5))
    assert(forecast.nem_time == datetime(2021, 5, 6, 12, 30, tzinfo=tzoffset(None, 36000)))
    assert(forecast.start_time == datetime(2021, 5, 5, 2, 0, 1, tzinfo=tzutc()))
    assert(forecast.end_time == datetime(2021, 5, 5, 2, 30, tzinfo=tzutc()))
    assert(forecast.renewables == 45.0)
    assert(forecast.channel_type == ChannelType.GENERAL)
    assert(forecast.spike_status == SpikeStatus.POTENTIAL)
    assert(forecast.tariff_information.period == PeriodType.SHOULDER)
    assert(forecast.tariff_information.season == SeasonType.WINTER)
    assert(forecast.tariff_information.block == 2)
    assert(forecast.tariff_information.demand_window == True)
    assert(forecast.range.min == 10)
    assert(forecast.range.max == 90)


def test_prices_params(mocker, configuration):
    def mock_prices(self, method, path, query_params=None, headers=None, body=None, post_params=None, _preload_content=True, _request_timeout=None):
        assert(method == "GET")
        assert(path == "https://api.amber.com.au/v1/sites/01F5A5CRKMZ5BCX9P1S4V990AM/prices")
        assert(query_params == {'startDate': '2021-05-05',
                                'endDate': '2021-05-06', 'resolution': '30'})
        assert(headers == {'Authorization': 'Bearer psk_secret_key'})
        return MockRestResponse(200, "", "[]".encode('utf-8'))

    mocker.patch('amberelectric.rest.RESTClientObject.request', mock_prices)

    api = AmberApi(RESTClientObject(configuration))
    api.get_prices('01F5A5CRKMZ5BCX9P1S4V990AM', start_date=date(
        2021, 5, 5), end_date=date(2021, 5, 6), resolution=30)


def test_current_price_success(mocker, configuration):
    json = """
      [
        {
          "type": "CurrentInterval",
          "duration": 5,
          "spotPerKwh": 6.12,
          "perKwh": 24.33,
          "date": "2021-05-05",
          "nemTime": "2021-05-06T12:30:00+10:00",
          "startTime": "2021-05-05T02:00:01Z",
          "endTime": "2021-05-05T02:30:00Z",
          "renewables": 45,
          "channelType": "general",
          "tariffInformation": {
            "period": "offPeak",
            "season": "summer",
            "block": 2,
            "demandWindow": true
          },
          "spikeStatus": "none",
          "range": {
            "min": 0,
            "max": 0
          },
          "estimate": true
        }
      ]
    """

    def mock_prices(self, method, path, query_params=None, headers=None, body=None, post_params=None, _preload_content=True, _request_timeout=None):
        assert(method == "GET")
        assert(path == "https://api.amber.com.au/v1/sites/01F5A5CRKMZ5BCX9P1S4V990AM/prices/current")
        assert(query_params is None)
        assert(headers == {'Authorization': 'Bearer psk_secret_key'})
        return MockRestResponse(200, "", json.encode('utf-8'))

    mocker.patch('amberelectric.rest.RESTClientObject.request', mock_prices)

    api = AmberApi(RESTClientObject(configuration))
    prices = api.get_current_price('01F5A5CRKMZ5BCX9P1S4V990AM')
    assert len(prices) == 1

    current = prices[0]

    assert(current.__class__ == CurrentInterval)
    assert(current.duration == 5)
    assert(current.spot_per_kwh == 6.12)
    assert(current.per_kwh == 24.33)
    assert(current.date == date(2021, 5, 5))
    assert(current.nem_time == datetime(2021, 5, 6, 12, 30, tzinfo=tzoffset(None, 36000)))
    assert(current.start_time == datetime(2021, 5, 5, 2, 0, 1, tzinfo=tzutc()))
    assert(current.end_time == datetime(2021, 5, 5, 2, 30, tzinfo=tzutc()))
    assert(current.renewables == 45.0)
    assert(current.channel_type == ChannelType.GENERAL)
    assert(current.spike_status == SpikeStatus.NO_SPIKE)
    assert(current.tariff_information.period == PeriodType.OFF_PEAK)
    assert(current.tariff_information.season == SeasonType.SUMMER)
    assert(current.tariff_information.block == 2)
    assert(current.tariff_information.demand_window == True)
    assert(current.range.min == 0)
    assert(current.range.max == 0)
    assert(current.estimate == True)


def test_current_prices_params(mocker, configuration):
    json = """
      [
        {
          "type": "ActualInterval",
          "duration": 5,
          "spotPerKwh": 6.12,
          "perKwh": 24.33,
          "date": "2021-05-05",
          "nemTime": "2021-05-06T12:00:00+10:00",
          "startTime": "2021-05-05T01:30:01Z",
          "endTime": "2021-05-05T02:00:00Z",
          "renewables": 45,
          "channelType": "general",
          "tariffInformation": {
            "period": "offPeak",
            "season": "summer",
            "block": 2,
            "demandWindow": true
          },
          "spikeStatus": "none"
        },
        {
          "type": "CurrentInterval",
          "duration": 5,
          "spotPerKwh": 6.12,
          "perKwh": 24.33,
          "date": "2021-05-05",
          "nemTime": "2021-05-06T12:30:00+10:00",
          "startTime": "2021-05-05T02:00:01Z",
          "endTime": "2021-05-05T02:30:00Z",
          "renewables": 45,
          "channelType": "general",
          "tariffInformation": {
            "period": "peak",
            "season": "weekend",
            "block": 2,
            "demandWindow": true
          },
          "spikeStatus": "spike",
          "range": {
            "min": 0,
            "max": 100
          },
          "estimate": true
        },
        {
          "type": "ForecastInterval",
          "duration": 5,
          "spotPerKwh": 6.12,
          "perKwh": 24.33,
          "date": "2021-05-05",
          "nemTime": "2021-05-06T01:30:00+10:00",
          "startTime": "2021-05-05T03:00:01Z",
          "endTime": "2021-05-05T03:30:00Z",
          "renewables": 45,
          "channelType": "general",
          "tariffInformation": {
            "period": "shoulder",
            "season": "winter",
            "block": 2,
            "demandWindow": true
          },
          "spikeStatus": "potential",
          "range": {
            "min": 10,
            "max": 90
          }
        }
      ]
    """

    def mock_prices(self, method, path, query_params=None, headers=None, body=None, post_params=None, _preload_content=True, _request_timeout=None):
        assert(method == "GET")
        assert(path == "https://api.amber.com.au/v1/sites/01F5A5CRKMZ5BCX9P1S4V990AM/prices/current")
        assert(query_params == {'resolution': '30', 'next': '1', 'previous': '1'})
        assert(headers == {'Authorization': 'Bearer psk_secret_key'})
        return MockRestResponse(200, "", json.encode('utf-8'))

    mocker.patch('amberelectric.rest.RESTClientObject.request', mock_prices)

    api = AmberApi(RESTClientObject(configuration))
    prices = api.get_current_price('01F5A5CRKMZ5BCX9P1S4V990AM', resolution=30, next=1, previous=1)
    assert len(prices) == 3
    assert(prices[0].__class__ == ActualInterval)
    assert(prices[1].__class__ == CurrentInterval)
    assert(prices[2].__class__ == ForecastInterval)


def test_usage_success(mocker, configuration):
    json = """
      [
        {
          "type": "Usage",
          "duration": 5,
          "spotPerKwh": 6.12,
          "perKwh": 24.33,
          "date": "2021-05-05",
          "nemTime": "2021-05-06T12:30:00+10:00",
          "startTime": "2021-05-05T02:00:01Z",
          "endTime": "2021-05-05T02:30:00Z",
          "renewables": 45,
          "channelType": "general",
          "tariffInformation": {
            "period": "offPeak",
            "season": "summer",
            "block": 2,
            "demandWindow": true
          },
          "spikeStatus": "none",
          "channelIdentifier": "E1",
          "kwh": 1,
          "quality": "estimated",
          "cost": 2
        }
      ]
    """

    def mock_prices(self, method, path, query_params=None, headers=None, body=None, post_params=None, _preload_content=True, _request_timeout=None):
        assert(method == "GET")
        assert(path == "https://api.amber.com.au/v1/sites/01F5A5CRKMZ5BCX9P1S4V990AM/usage")
        assert(query_params == {'startDate': '2021-05-05', 'endDate': '2021-05-06'})
        assert(headers == {'Authorization': 'Bearer psk_secret_key'})
        return MockRestResponse(200, "", json.encode('utf-8'))

    mocker.patch('amberelectric.rest.RESTClientObject.request', mock_prices)

    api = AmberApi(RESTClientObject(configuration))
    prices = api.get_usage('01F5A5CRKMZ5BCX9P1S4V990AM', date(2021, 5, 5), date(2021, 5, 6))
    assert len(prices) == 1

    usage = prices[0]

    assert(usage.__class__ == Usage)
    assert(usage.duration == 5)
    assert(usage.spot_per_kwh == 6.12)
    assert(usage.per_kwh == 24.33)
    assert(usage.date == date(2021, 5, 5))
    assert(usage.nem_time == datetime(2021, 5, 6, 12, 30, tzinfo=tzoffset(None, 36000)))
    assert(usage.start_time == datetime(2021, 5, 5, 2, 0, 1, tzinfo=tzutc()))
    assert(usage.end_time == datetime(2021, 5, 5, 2, 30, tzinfo=tzutc()))
    assert(usage.renewables == 45.0)
    assert(usage.channel_type == ChannelType.GENERAL)
    assert(usage.spike_status == SpikeStatus.NO_SPIKE)
    assert(usage.tariff_information.period == PeriodType.OFF_PEAK)
    assert(usage.tariff_information.season == SeasonType.SUMMER)
    assert(usage.tariff_information.block == 2)
    assert(usage.tariff_information.demand_window == True)
    assert(usage.kwh == 1)
    assert(usage.quality == 'estimated')
    assert(usage.cost == 2)
    assert(usage.channelIdentifier == "E1")


def test_usage_success_params(mocker, configuration):
    def mock_prices(self, method, path, query_params=None, headers=None, body=None, post_params=None, _preload_content=True, _request_timeout=None):
        assert(method == "GET")
        assert(path == "https://api.amber.com.au/v1/sites/01F5A5CRKMZ5BCX9P1S4V990AM/usage")
        assert(query_params == {'startDate': '2021-05-05',
                                'endDate': '2021-05-06', 'resolution': '30'})
        assert(headers == {'Authorization': 'Bearer psk_secret_key'})
        return MockRestResponse(200, "", "[]".encode('utf-8'))

    mocker.patch('amberelectric.rest.RESTClientObject.request', mock_prices)

    api = AmberApi(RESTClientObject(configuration))
    api.get_usage('01F5A5CRKMZ5BCX9P1S4V990AM', date(2021, 5, 5), date(2021, 5, 6), resolution=30)



from typing import Union
from datetime import date, datetime
from .interval import Interval
from .range import Range


class Usage(Interval):
    channelIdentifier: str
    kwh: float
    quality: str
    cost: float

    def __init__(
        self,
        duration: float,
        spot_per_kwh: float,
        per_kwh: float,
        date: date,
        nem_time: datetime,
        start_time: datetime,
        end_time: datetime,
        renewables: float,
        channel_type: str,
        spike_status: str,
        channelIdentifier: str,
        kwh: float,
        quality: str,
        cost: float,
        **kwargs
    ):
        super().__init__(
            duration,
            spot_per_kwh,
            per_kwh,
            date,
            nem_time,
            start_time,
            end_time,
            renewables,
            channel_type,
            spike_status,
            **kwargs)

        self.channelIdentifier = channelIdentifier
        self.kwh = kwh
        self.quality = quality
        self.cost = cost

    def to_dict(self) -> dict:
        d = super().to_dict()
        d.update({
            "type": "Usage",
            "channelIdentifier": self.channelIdentifier,
            "kwh": self.kwh,
            "quality": self.quality,
            "cost": self.cost
        })
        return d



class Range(object):
    """When prices are particularly volatile, the API may return a range of prices that are possible."""

    """Estimated minimum price (c/kWh)"""
    min: float
    """Estimated maximum price (c/kWh)"""
    max: float

    def __init__(self, min: float, max: float):
        self.min = min
        self.max = max

    def __repr__(self) -> str:
        return self.to_str()

    def to_str(self):
        return str({'min': self.min, 'max': self.max})



from .interval import Interval


class ActualInterval(Interval):
    def __init__(
        self,
        duration,
        spot_per_kwh,
        per_kwh,
        date,
        nem_time,
        start_time,
        end_time,
        renewables,
        channel_type,
        spike_status,
        **kwargs
    ):
        super().__init__(
            duration,
            spot_per_kwh,
            per_kwh,
            date,
            nem_time,
            start_time,
            end_time,
            renewables,
            channel_type,
            spike_status,
            **kwargs)

    def to_dict(self) -> dict:
        d = super().to_dict()
        d.update({
            "type": "ActualInterval"
        })
        return d



# flake8: noqa

"""
    Amber Electric Public API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0
    Generated by: https://openapi-generator.tech
"""


__version__ = "1.0.0"

# import Configuration
from amberelectric.configuration import Configuration

# import exceptions
from amberelectric.exceptions import OpenApiException
from amberelectric.exceptions import ApiAttributeError
from amberelectric.exceptions import ApiTypeError
from amberelectric.exceptions import ApiValueError
from amberelectric.exceptions import ApiKeyError
from amberelectric.exceptions import ApiException



from typing import List, Optional, Union
import json
import dateutil.parser
from datetime import date

from amberelectric.rest import RESTResponse
from amberelectric.configuration import Configuration
from amberelectric.model.site import Site
from amberelectric.model.current_interval import CurrentInterval
from amberelectric.model.actual_interval import ActualInterval
from amberelectric.model.forecast_interval import ForecastInterval
from amberelectric.model.usage import Usage
from amberelectric.model.tariff_information import TariffInformation
from amberelectric.model.channel import Channel
from amberelectric.model.range import Range
from amberelectric.exceptions import ApiException

from ..rest import RESTClientObject


def parse_tariff_information(tariff_information: Optional[object]) -> TariffInformation:
    if tariff_information is None:
        return None
    else:
        return TariffInformation(**tariff_information)


def parse_range(range: Optional[object]) -> Range:
    if range is None:
        return None
    else:
        return Range(float(range['min']), float(range['max']))


def parse_interval(interval: object) -> Union[ActualInterval, CurrentInterval, ForecastInterval, Usage]:
    optional = {}

    if 'tariffInformation' in interval:
        optional['tariff_information'] = parse_tariff_information(interval['tariffInformation'])

    if 'range' in interval:
        optional['range'] = parse_range(interval['range'])

    if interval['type'] == 'ActualInterval':
        return ActualInterval(
            float(interval['duration']),
            float(interval['spotPerKwh']),
            float(interval['perKwh']),
            dateutil.parser.isoparse(interval['date']).date(),
            dateutil.parser.isoparse(interval['nemTime']),
            dateutil.parser.isoparse(interval['startTime']),
            dateutil.parser.isoparse(interval['endTime']),
            float(interval['renewables']),
            interval['channelType'],
            interval['spikeStatus'],
            **optional
        )

    if interval['type'] == 'CurrentInterval':
        return CurrentInterval(
            float(interval['duration']),
            float(interval['spotPerKwh']),
            float(interval['perKwh']),
            dateutil.parser.isoparse(interval['date']).date(),
            dateutil.parser.isoparse(interval['nemTime']),
            dateutil.parser.isoparse(interval['startTime']),
            dateutil.parser.isoparse(interval['endTime']),
            float(interval['renewables']),
            interval['channelType'],
            interval['spikeStatus'],
            interval['estimate'],
            **optional
        )

    if interval['type'] == 'ForecastInterval':
        return ForecastInterval(
            float(interval['duration']),
            float(interval['spotPerKwh']),
            float(interval['perKwh']),
            dateutil.parser.isoparse(interval['date']).date(),
            dateutil.parser.isoparse(interval['nemTime']),
            dateutil.parser.isoparse(interval['startTime']),
            dateutil.parser.isoparse(interval['endTime']),
            float(interval['renewables']),
            interval['channelType'],
            interval['spikeStatus'],
            **optional
        )

    if interval['type'] == 'Usage':
        return Usage(
            float(interval['duration']),
            float(interval['spotPerKwh']),
            float(interval['perKwh']),
            dateutil.parser.isoparse(interval['date']).date(),
            dateutil.parser.isoparse(interval['nemTime']),
            dateutil.parser.isoparse(interval['startTime']),
            dateutil.parser.isoparse(interval['endTime']),
            float(interval['renewables']),
            interval['channelType'],
            interval['spikeStatus'],
            interval['channelIdentifier'],
            float(interval['kwh']),
            interval['quality'],
            float(interval['cost']),
            **optional
        )


def parse_intervals(intervals: List[object]) -> List[Union[ActualInterval, CurrentInterval, ForecastInterval, Usage]]:
    return list(map(parse_interval, intervals))


def parse_channel(channel: object) -> Channel:
    return Channel(channel['identifier'], channel['type'])


def parse_channels(channels: List[object]) -> List[Channel]:
    return list(map(parse_channel, channels))


def parse_site(site: object) -> Site:
    return Site(site['id'], site['nmi'], parse_channels(site['channels']))


def parse_sites(sites: List[object]) -> List[Site]:
    return list(map(parse_site, sites))


class AmberApi:
    def __init__(self, rest_client: RESTClientObject):
        self._rest_client = rest_client

    def create(configuration: Configuration):
        return AmberApi(RESTClientObject(configuration))

    def request(self, method, path, query_params=None, headers=None,
                body=None, post_params=None, _preload_content=True,
                _request_timeout=None) -> RESTResponse:
        auth = self._rest_client.configuration.auth_settings()
        if 'apiKey' in auth:
            token = auth['apiKey']
            if headers is None:
                headers = {}
            headers[token['key']] = token['value']

        url = self._rest_client.configuration.host + path
        return self._rest_client.request(method, url, query_params, headers, body, post_params, _preload_content, _request_timeout)

    def get_sites(self) -> List[Site]:
        response = self.request("GET", "/sites")
        if response.status == 200:
            return parse_sites(json.loads(response.data.decode("utf-8")))
        else:
            raise ApiException(response.status, response.reason, response)

    def get_current_price(self, site_id: str, **kwargs) -> List[Union[ActualInterval, CurrentInterval, ForecastInterval]]:
        query_params = {}
        if "resolution" in kwargs:
            query_params["resolution"] = str(kwargs.get("resolution"))
        if "previous" in kwargs:
            query_params["previous"] = str(kwargs.get("previous"))
        if "next" in kwargs:
            query_params["next"] = str(kwargs.get("next"))

        if query_params == {}:
            query_params = None
        response = self.request("GET", "/sites/" + site_id + "/prices/current", query_params)

        if response.status == 200:
            return parse_intervals(json.loads(response.data.decode("utf-8")))
        else:
            raise ApiException(response.status, response.reason, response)

    def get_prices(self, site_id: str, **kwargs) -> List[Union[ActualInterval, CurrentInterval, ForecastInterval]]:
        query_params = {}
        if "end_date" in kwargs:
            query_params["endDate"] = kwargs.get("end_date").isoformat()
        if "start_date" in kwargs:
            query_params["startDate"] = kwargs.get("start_date").isoformat()
        if "resolution" in kwargs:
            query_params["resolution"] = str(kwargs.get("resolution"))

        if query_params == {}:
            query_params = None

        response = self.request("GET", "/sites/" + site_id + "/prices", query_params)

        if response.status == 200:
            return parse_intervals(json.loads(response.data.decode("utf-8")))
        else:
            raise ApiException(response.status, response.reason, response)

    def get_usage(self, site_id: str, start_date: date, end_date: date, **kwargs) -> Usage:
        query_params = {'startDate': start_date.isoformat(), 'endDate': end_date.isoformat()}
        if "resolution" in kwargs:
            query_params["resolution"] = str(kwargs.get("resolution"))

        response = self.request("GET", "/sites/" + site_id + "/usage", query_params)

        if response.status == 200:
            return parse_intervals(json.loads(response.data.decode("utf-8")))
        else:
            raise ApiException(response.status, response.reason, response)



from typing import Union
from enum import Enum


class PeriodType(Enum):
    OFF_PEAK = "offPeak"
    SHOULDER = "shoulder"
    SOLAR_SPONGE = "solarSponge"
    PEAK = "peak"

    def from_str(s: Union[str, None]):
        possible = list(filter(lambda t: t.value == s, PeriodType))
        if len(possible) > 0:
            return possible[0]


class SeasonType(Enum):
    SUMMER = "summer"
    AUTUMN = "autumn"
    WINTER = "winter"
    SPRING = "spring"
    NON_SUMMER = "nonSummer"
    HOLIDAY = "holiday"
    WEEKEND = "weekend"
    WEEKEND_HOLIDAY = "weekendHoliday"
    WEEKDAY = "weekday"

    def from_str(s: Union[str, None]):
        possible = list(filter(lambda t: t.value == s, SeasonType))
        if len(possible) > 0:
            return possible[0]


class TariffInformation(object):
    """Information about how your tariff affects an interval"""

    """The Time of Use period that is currently active. Only available if the site in on a time of use tariff"""
    period: Union[PeriodType, None]

    """The Time of Use season that is currently active. Only available if the site in on a time of use tariff"""
    season: Union[SeasonType, None]

    """The block that is currently active. Only available in the site in on a block tariff"""
    block: Union[int, None]

    """Is this interval currently in the demand window? Only available if the site in on a demand tariff"""
    demand_window: Union[bool, None]

    def __init__(self, **kwargs):
        self.period = PeriodType.from_str(kwargs.get('period'))
        self.season = SeasonType.from_str(kwargs.get('season'))
        self.block = kwargs.get('block')
        self.demand_window = kwargs.get('demandWindow')

    def __repr__(self) -> str:
        return self.to_str()

    def to_str(self):
        return str({'period': self.period, 'season': self.season, 'block': self.block, 'demand_window': self.demand_window})



from enum import Enum
from typing import Union


class ChannelType(Enum):
    GENERAL = "general"
    CONTROLLED_LOAD = "controlledLoad"
    FEED_IN = "feedIn"

    def from_str(s: Union[str, None]):
        possible = list(filter(lambda t: t.value == s, ChannelType))
        if len(possible) > 0:
            return possible[0]


class Channel(object):
    """Describes a power meter channel.

    The General channel provides continuous power - it's the channel all of your appliances and lights are attached to.

    Controlled loads are only on for a limited time during the day (usually when the load on the network is low, or generation is high) - you may have your hot water system attached to this channel.

    The feed in channel sends power back to the grid - you will have these types of channels if you have solar or batteries."""

    """Identifier of the channel"""
    identifier: str

    """Channel type."""
    type: ChannelType

    def __init__(self, identifier: str, type: ChannelType):
        self.identifier = identifier
        self.type = ChannelType.from_str(type)

    def __repr__(self) -> str:
        return self.to_str()

    def to_str(self):
        return str({'identifier': self.identifier, 'type': self.type})



from typing import List
from .channel import Channel


class Site(object):
    def __init__(self, id: str, nmi: str, channels: List[Channel]):
        self.id = id
        self.nmi = nmi
        self.channels = channels

    def __repr__(self) -> str:
        return self.to_str()

    def to_str(self):
        return str({'id': self.id, 'nmi': self.nmi, 'channels': self.channels})



from datetime import date, datetime
from enum import Enum
from typing import Union
from .tariff_information import TariffInformation
from .channel import ChannelType


class SpikeStatus(Enum):
    NO_SPIKE = "none"
    POTENTIAL = "potential"
    SPIKE = "spike"

    def from_str(s: Union[str, None]):
        possible = list(filter(lambda t: t.value == s, SpikeStatus))
        if len(possible) > 0:
            return possible[0]


class Interval(object):
    """Length of the interval in minutes."""
    duration: str
    """NEM spot price. This is the price generators get paid to generate electricity, and what drives the variable component of your perKwh price"""
    spot_per_kwh: float
    """Number of cents you will pay per kilowatt-hour (c/kWh)"""
    per_kwh: float
    """Date the interval belongs to. This may be different to the date component of nemTime, as the last interval of the day ends at 12:30 the following day. Formatted as a ISO 8601 date"""
    date: date
    """The interval's NEM time. This represents the time at the end of the interval UTC+10. Formatted as a ISO 8601 time"""
    nem_time: datetime
    """Start time of the interval in UTC. Formatted as a ISO 8601 time"""
    start_time: datetime
    """End time of the interval in UTC. Formatted as a ISO 8601 time"""
    end_time: datetime
    """Percentage of renewables in the grid"""
    renewables: float
    """Meter channel type"""
    channel_type: str
    """Indicates whether this interval will potentially spike, or is currently in a spike state"""
    spike_status: str
    tariff_information: TariffInformation

    def __init__(
        self,
        duration: float,
        spot_per_kwh: float,
        per_kwh: float,
        date: date,
        nem_time: datetime,
        start_time: datetime,
        end_time: datetime,
        renewables: float,
        channel_type: str,
        spike_status: str,
        **kwargs
    ):
        self.duration = duration
        self.spot_per_kwh = spot_per_kwh
        self.per_kwh = per_kwh
        self.date = date
        self.nem_time = nem_time
        self.start_time = start_time
        self.end_time = end_time
        self.renewables = renewables
        self.channel_type = ChannelType.from_str(channel_type)
        self.spike_status = SpikeStatus.from_str(spike_status)
        self.tariff_information = kwargs.get('tariff_information')

    def to_dict(self) -> dict:
        return {
            "duration": self.duration,
            "spot_per_kwh": self.spot_per_kwh,
            "per_kwh": self.per_kwh,
            "date": self.date,
            "nem_time": self.nem_time,
            "start_time": self.start_time,
            "end_time": self.end_time,
            "renewables": self.renewables,
            "channel_type": self.channel_type,
            "spike_status": self.spike_status,
        }

    def __repr__(self) -> str:
        return self.to_str()

    def to_str(self):
        return str(self.to_dict())



from setuptools import setup, find_packages  # noqa: H301

NAME = "amberelectric.py"
VERSION = "1.0.0"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = [
    "urllib3 >= 1.25.3",
    "python-dateutil",
]

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name=NAME,
    version=VERSION,
    author="Myles Eftos",
    author_email="myles@madpilot.com.au",
    url="https://github.com/madpilot/amberelectric.py",
    keywords=["amberelectric"],
    python_requires=">=3.6",
    install_requires=REQUIRES,
    packages=find_packages(exclude=["test", "tests"]),
    include_package_data=True,
    description="Interface to the Amber Electric API, allowing you to download current and forecast price, as well as download your historic usage.",
    long_description=long_description,
    long_description_content_type="text/markdown",
)



from typing import Union
from datetime import date, datetime
from .interval import Interval
from .range import Range


class CurrentInterval(Interval):
    range: Union[Range, None]
    """Shows true the current price is an estimate. Shows false is the price has been locked in."""
    estimate: bool

    def __init__(
        self,
        duration: float,
        spot_per_kwh: float,
        per_kwh: float,
        date: date,
        nem_time: datetime,
        start_time: datetime,
        end_time: datetime,
        renewables: float,
        channel_type: str,
        spike_status: str,
        estimate: bool,
        **kwargs
    ):
        super().__init__(
            duration,
            spot_per_kwh,
            per_kwh,
            date,
            nem_time,
            start_time,
            end_time,
            renewables,
            channel_type,
            spike_status,
            **kwargs)

        self.estimate = estimate
        self.range = kwargs.get('range')

    def to_dict(self) -> dict:
        d = super().to_dict()
        d.update({
            "type": "CurrentInterval",
            "range": self.range,
            "estimate": self.estimate
        })
        return d
